moXmlSettings = {}

function moXmlSettings:load(self, serverXml, languageXml, objectXml, clientXml)
	local file;
	function checkFile(xmlType, file, tableType)
		if not fileExists(file) then 
			moXmlSettings:generateXml(self, xmlType, file, tableType);
		else
			moXmlSettings:loadXml(self, xmlType, file, tableType);
		end;
	end;	
	if MultiOverlayV2.loadDetiServer or MultiOverlayV2.loadHostServer and serverXml then
		file = MultiOverlayV2.settingsDir.. "serverSettings.xml";
		if MultiOverlayV2.loadDetiServer then file = MultiOverlayV2.detiServerDir.. "serverSettings.xml";end;
		checkFile("serverSettings", file);
	end;
	if not MultiOverlayV2.loadDetiServer or MultiOverlayV2.loadHostServer then		
		if languageXml then
			file = MultiOverlayV2.settingsDir.. "language".. MultiOverlayV2.language.. ".xml";		
			saveFile = MultiOverlayV2.saveGameDir.. "language".. MultiOverlayV2.language.. ".xml";
			if fileExists(saveFile) then file = saveFile;moGeneralSettings.saveGameLanguage = true;end;
			checkFile("language", file, moLanguageDefault);
		end;
		if clientXml then
			file = MultiOverlayV2.settingsDir.. "clientSettings.xml";
			saveFile = MultiOverlayV2.saveGameDir.. "clientSettings.xml";
			if fileExists(saveFile) then file = saveFile;moGeneralSettings.saveGameClientSettings = true;end;
			checkFile("clientSettings", file);
		end;
		if objectXml then
			file = MultiOverlayV2.settingsDir.. "objectSettings.xml";
			saveFile = MultiOverlayV2.saveGameDir.. "objectSettings.xml";
			if fileExists(saveFile) then file = saveFile;moGeneralSettings.saveGameObjectSettings = true;end;
			checkFile("objectSettings", file);
		end;
	end;	
end;

function moXmlSettings:generateXml(self, xmlType, file, tableType, replace)
	local txt = "";
	if xmlType == "language" then txt = MultiOverlayV2.language;end;
	if replace then
		print(tostring(MultiOverlayV2.metadata.title).. " replace ".. tostring(xmlType).. tostring(txt).. " xml file")
	else
		print(tostring(MultiOverlayV2.metadata.title).. " generated ".. tostring(xmlType).. tostring(txt).. " xml file")
	end;		
	local a = 0;	
	if xmlType == "language" then
		local Xml = createXMLFile("MultiOverlayV2_XML", file, xmlType);
		for name, v in pairs(tableType) do
			if name ~= nil and tableType[name] ~= nil and tableType[name][1] ~= nil then
				local xmlNameTag = ("language.text(%d)"):format(a);
				--setXMLInt(Xml, xmlNameTag.."#id", a);
				setXMLString(Xml, xmlNameTag.."#name", name);
				setXMLString(Xml, xmlNameTag.."#text", tableType[name][1]);			
				a = a+1;
			end;
		end;		
		if not replace then moLanguage = tableType;end;
		saveXMLFile(Xml);
		return;
	end;
	if xmlType == "serverSettings" then		
		local Xml = createXMLFile("MultiOverlayV2_XML", file, xmlType);
		local xmlNameTag = ("serverSettings.values(%d)"):format(a);
		--setXMLInt(Xml, xmlNameTag.."#id", a);
		setXMLBool(Xml, xmlNameTag.."#greatDemandPage", false);
		setXMLBool(Xml, xmlNameTag.."#discountPage", false);		
		setXMLBool(Xml, xmlNameTag.."#infoPage", false);
		setXMLInt(Xml, xmlNameTag.."#infoEditUser", 1);
		setXMLString(Xml, xmlNameTag.."#infoEditPw", "1234");
		setXMLString(Xml, xmlNameTag.."#infoEditMasterPw", "1234");
		setXMLBool(Xml, xmlNameTag.."#saveWeather", false);
		saveXMLFile(Xml);		
		return;		
	end;
	if xmlType == "objectSettings" then
		local Xml = createXMLFile("MultiOverlayV2_XML", file, xmlType);
		local version = string.format("%.1f", Utils.getNoNil(MultiOverlayV2.metadata.versionXml, 0.0));
		setXMLString(Xml, xmlType.. ".info.version", tostring(version));
		setXMLString(Xml, xmlType.. ".info.acceptedModTypes", "upk fs sr ls other(upk = upk mods,fs = marhu fabrikscript mods,sr = marhu schweinerinder mods,ls = LS15,other = not free)");
		setXMLString(Xml, xmlType.. ".info.acceptedSlotType", "animals greenhouse trees factory1 factory2 factory3 other1 other2 stocks1 stocks2 shops1 shops2" );
		setXMLString(Xml, xmlType.. ".info.acceptedXmlString", "xmlString=(zip name/xml file.xml or mapObject name");		
		setXMLString(Xml, xmlType.. ".info.acceptedCategory", "factory trees stocks heaps shops animals (default = factory)");
		setXMLString(Xml, xmlType.. ".info.acceptedBooleanValues", "separated=(true or false) and isHiden=(true or false) (default = false)");
		setXMLString(Xml, xmlType.. ".info.acceptedOther", "displayName=(length max 30) (default = factory)");
		setXMLString(Xml, xmlType.. ".info.required", "modType=,slotType=,xmlString=");				
		function setXml(tagString, tableObject, map)		
			a = 0;
			function setTable(tableObject, modType, slotType, xmlString, category, separated, isHiden, displayName)
				if modType == "fs" and map then 
					moObjectsMapFsFoundXml = moObjectsMapFsFoundXml.. tostring(xmlString).. " ";				
				end;
				if modType == "other" and map then 
					moObjectsMapOtherFoundXml = moObjectsMapOtherFoundXml.. tostring(xmlString).. " ";				
				end;
				tableObject[slotType.. "Found"] = tableObject[slotType.. "Found"]+1;
				table.insert(tableObject[slotType], modType.. ",".. xmlString.. ",".. category.. ",".. tostring(separated).. ",".. tostring(isHiden).. ",".. displayName);				
			end;
			for k, v in pairs(tableObject) do
				local xmlNameTag = (tagString):format(a);
				local splitString = Utils.splitString(",", v);				
				--setXMLInt(Xml, xmlNameTag.."#id", a);
				setXMLString(Xml, xmlNameTag.."#modType", splitString[1]);
				setXMLString(Xml, xmlNameTag.."#slotType", splitString[2]);
				setXMLString(Xml, xmlNameTag.."#xmlString", splitString[3]);
				setXMLString(Xml, xmlNameTag.."#category", splitString[4]);
				local boolean = false;
				if splitString[5] == "true" then boolean = true;end;
				setXMLBool(Xml, xmlNameTag.."#separated", boolean);						
				boolean = false;
				if splitString[6] == "true" then boolean = true;end;
				setXMLBool(Xml, xmlNameTag.."#isHiden", boolean);
				local replaceName = splitString[7];
				if string.len(replaceName) > 30 then replaceName = string.sub (splitString[7], 0, 30);end;
				setXMLString(Xml, xmlNameTag.."#displayName", tostring(replaceName));		
				a = a+1;
				if map then setTable(moObjectsMap, splitString[1], splitString[2], splitString[3], splitString[4], splitString[5], splitString[6], replaceName);end;
				if not map then setTable(moObjectsPlaceable, splitString[1], splitString[2], splitString[3], splitString[4], splitString[5], splitString[6], replaceName);end
			end;
		end;
		setXml(xmlType.. ".map.mod(%d)", moObjectsDefault.map, true);		
		setXml(xmlType.. ".placeable.mod(%d)", moObjectsDefault.placeable, false);
		saveXMLFile(Xml);				
		return;		
	end;
	if xmlType == "clientSettings" then			
		moSetGetConfigXml:saveConfig(self);
		return;		
	end;
end;

function moXmlSettings:loadXml(self, xmlType, file, tableType)
	local txt = "";
	local saveGame = "";
	if xmlType == "clientSettings" and moGeneralSettings.saveGameClientSettings then saveGame = " (savegame".. g_currentMission.missionInfo.savegameIndex.. ")";end;
	if xmlType == "objectSettings" and moGeneralSettings.saveGameObjectSettings then saveGame = " (savegame".. g_currentMission.missionInfo.savegameIndex.. ")";end;
	if xmlType == "language" then txt = MultiOverlayV2.language;end;
	print(tostring(MultiOverlayV2.metadata.title).. " load ".. tostring(xmlType).. tostring(txt).. " xml file".. tostring(saveGame))
	Xml = loadXMLFile("MultiOverlayV2_XML", file, xmlType);
	local xmlReplace = false;
	local a=0;
	if xmlType == "language" then
		while true do			
			local xmlNameTag = ("language.text(%d)"):format(a);			
			if getXMLString(Xml, xmlNameTag.."#name") and getXMLString(Xml, xmlNameTag.."#text") then
				local name = string.lower(getXMLString(Xml, xmlNameTag.."#name"));
				local text = getXMLString(Xml, xmlNameTag.."#text");
				if moLanguageDefault[name] ~= nil then table.remove(moLanguageDefault[name]);end;				
				moLanguage[name] = {text};
			else
				break;
			end;
			a = a+1;			
		end;		
		for name, v in pairs(tableType) do
			if name ~= nil and tableType[name] ~= nil and tableType[name][1] ~= nil then				
				moLanguage[name] = {tableType[name][1]};
				--print("sprache replace.....".. tostring(name))
				xmlReplace = true;
			end;
		end;		
		if xmlReplace then 
			moXmlSettings:generateXml(self, xmlType, file, moLanguage, true);			
		end;
		return;		
	end;
	if xmlType == "objectSettings" then		
		--local acceptedOther = "mischstation ruebenschneider kompostier_anlage seedmaster2k15 beetmaster100k ";
		local acceptedSlotType = "animals greenhouse trees factory1 factory2 factory3 other1 other2 stocks1 stocks2 shops1 shops2 ";
		local acceptedCategory = "factory trees shops stocks heaps animals ";
		local acceptedModType = "upk fs ls sr other ";
		function getXml(tagString, map)
			a = 0;
			function checkXmlType(modType, xmlString, map)
				if modType ~= "other" or map then return true;end;	
				--if modType == "other" and not map and string.find(xmlString, "mischstation") then return true;end;
				--if modType == "other" and not map and string.find(xmlString, "ruebenschneider") then return true;end;
				if modType == "other" and not map and string.find(xmlString, "heaptiptrigger") then return true;end;			
				--if string.find(moObjectsDefault.acceptOther, xmlString.. " ") then return true;end;
				return false;
			end;
			function setTable(tableObject, modType, slotType, xmlString, category, separated, isHiden, displayName)
				if modType == "fs" and map then 
					moObjectsMapFsFoundXml = moObjectsMapFsFoundXml.. tostring(xmlString).. " ";				
				end;
				if modType == "other" and map then 
					moObjectsMapOtherFoundXml = moObjectsMapOtherFoundXml.. tostring(xmlString).. " ";				
				end;
				tableObject[slotType.. "Found"] = tableObject[slotType.. "Found"]+1
				table.insert(tableObject[slotType], modType.. ",".. xmlString.. ",".. category.. ",".. tostring(separated).. ",".. tostring(isHiden).. ",".. displayName);				
			end;
			while true do			
				local xmlNameTag = (tagString):format(a);			
				if xmlNameTag == nil then break;end;
				if getXMLString(Xml, xmlNameTag.."#modType") and getXMLString(Xml, xmlNameTag.."#slotType") and getXMLString(Xml, xmlNameTag.."#xmlString") then					
					local slotType = string.lower(getXMLString(Xml, xmlNameTag.."#slotType"));
					if string.find(acceptedSlotType, slotType.. " ") then
						local xmlString = string.lower(getXMLString(Xml, xmlNameTag.."#xmlString"));
						local modType = string.lower(getXMLString(Xml, xmlNameTag.."#modType"));
						if string.find(acceptedModType, modType.. " ") then
							local xmlOkay = true;
							if string.find(xmlString, "heaptiptrigger") and modType ~= "other" then xmlOkay = false;end;
							if xmlOkay then --checkXmlType(modType, xmlString, map) then								
								local displayName = modType.. "P".. a+1;						
								if map then displayName = modType.. "M".. a+1;end;
								local category = "factory";
								local separated = false;
								local isHiden = false;
								if getXMLString(Xml, xmlNameTag.."#displayName") then 
									displayName = getXMLString(Xml, xmlNameTag.."#displayName");
									local replaceName = displayName;
									if string.len(replaceName) > 30 then replaceName = string.sub (displayName, 0, 30);end;
									displayName = replaceName;								
								end;
								if getXMLBool(Xml, xmlNameTag.."#separated") ~= nil then separated = getXMLBool(Xml, xmlNameTag.."#separated");end;
								if getXMLBool(Xml, xmlNameTag.."#isHiden") ~= nil then isHiden = getXMLBool(Xml, xmlNameTag.."#isHiden");end;
								if getXMLString(Xml, xmlNameTag.."#category") then 
									category = getXMLString(Xml, xmlNameTag.."#category");									
									if not string.find(acceptedCategory, category.. " ") then category = "factory";end;
									if string.find(xmlString, "placeableheap") or string.find(xmlString, "heaptiptrigger") then category = "heaps";separated=true;end;
								end;									
								if map then setTable(moObjectsMap, modType, slotType, xmlString, category, separated, isHiden, displayName);end;
								if not map then setTable(moObjectsPlaceable, modType, slotType, xmlString, category, separated, isHiden, displayName);end
							end;
						end;
					end;				
				end;
				a = a+1;
				local xmlNameTagTemp = (tagString):format(a);
				if getXMLString(Xml, xmlNameTagTemp.."#modType") == nil then break;end;
			end;
		end;
		getXml(xmlType.. ".map.mod(%d)", true);				
		getXml(xmlType.. ".placeable.mod(%d)", false);
		return;
	end;
	if xmlType == "serverSettings" then
		local xmlNameTag = ("serverSettings.values(%d)"):format(0);
		if getXMLBool(Xml, xmlNameTag.."#greatDemandPage") ~= nil then MultiOverlayV2.event.greatDemand_s = getXMLBool(Xml, xmlNameTag.."#greatDemandPage");end;
		if getXMLBool(Xml, xmlNameTag.."#discountPage") ~= nil then MultiOverlayV2.event.discount_s = getXMLBool(Xml, xmlNameTag.."#discountPage");end;
		if getXMLBool(Xml, xmlNameTag.."#infoPage") ~= nil and getXMLInt(Xml, xmlNameTag.."#infoEditUser") ~= nil and getXMLString(Xml, xmlNameTag.."#infoEditPw") ~= nil and getXMLString(Xml, xmlNameTag.."#infoEditMasterPw") ~= nil then 
			MultiOverlayV2.event.info_s = getXMLBool(Xml, xmlNameTag.."#infoPage");
			MultiOverlayV2.event.infoEditUser_s = getXMLInt(Xml, xmlNameTag.."#infoEditUser");
			MultiOverlayV2.event.infoEditPw_s = getXMLString(Xml, xmlNameTag.."#infoEditPw");
			MultiOverlayV2.event.infoEditMasterPw_s = getXMLString(Xml, xmlNameTag.."#infoEditMasterPw");
			if string.len(MultiOverlayV2.event.infoEditPw_s) < 1 then MultiOverlayV2.event.infoEditPw_s = "1234";end;
			if string.len(MultiOverlayV2.event.infoEditMasterPw_s) < 1 then MultiOverlayV2.event.infoEditMasterPw_s = "1234";end;
			if MultiOverlayV2.event.infoEditUser_s > 4 then MultiOverlayV2.event.infoEditUser_s = 1;end;
		else
			local Xml = createXMLFile("MultiOverlayV2_XML", file, xmlType);
			local xmlNameTag = ("serverSettings.values(%d)"):format(a);
			setXMLBool(Xml, xmlNameTag.."#greatDemandPage", MultiOverlayV2.event.greatDemand_s);
			setXMLBool(Xml, xmlNameTag.."#discountPage", MultiOverlayV2.event.discount_s);
			setXMLBool(Xml, xmlNameTag.."#infoPage", MultiOverlayV2.event.info_s);
			setXMLInt(Xml, xmlNameTag.."#infoEditUser", MultiOverlayV2.event.infoEditUser_s);
			setXMLString(Xml, xmlNameTag.."#infoEditMasterPw", MultiOverlayV2.event.infoEditMasterPw_s);
			setXMLString(Xml, xmlNameTag.."#infoEditPw", MultiOverlayV2.event.infoEditPw_s);
			saveXMLFile(Xml);
			a = 0;
		end;
		if getXMLBool(Xml, xmlNameTag.."#saveWeather") ~= nil then
			MultiOverlayV2.event.saveWeather_s = getXMLBool(Xml, xmlNameTag.."#saveWeather");
		else
			local Xml = createXMLFile("MultiOverlayV2_XML", file, xmlType);
			local xmlNameTag = ("serverSettings.values(%d)"):format(a);
			setXMLBool(Xml, xmlNameTag.."#greatDemandPage", MultiOverlayV2.event.greatDemand_s);
			setXMLBool(Xml, xmlNameTag.."#discountPage", MultiOverlayV2.event.discount_s);
			setXMLBool(Xml, xmlNameTag.."#infoPage", MultiOverlayV2.event.info_s);
			setXMLInt(Xml, xmlNameTag.."#infoEditUser", MultiOverlayV2.event.infoEditUser_s);
			setXMLString(Xml, xmlNameTag.."#infoEditPw", MultiOverlayV2.event.infoEditPw_s);
			setXMLString(Xml, xmlNameTag.."#infoEditMasterPw", MultiOverlayV2.event.infoEditMasterPw_s);
			setXMLBool(Xml, xmlNameTag.."#saveWeather", MultiOverlayV2.event.saveWeather_s);
			saveXMLFile(Xml);
			a = 0;
		end;
		return;
	end;
	if xmlType == "clientSettings" then		
		moSetGetConfigXml:loadConfig(self, Xml, xmlType)
		return;		
	end;
end;

function moXmlSettings:generateSavegameXml(self, xmlType)
	local fileCopy;
	local saveFile;
	if xmlType == "clientSettings" then
		if not fileExists(MultiOverlayV2.saveGameDir) then createFolder(MultiOverlayV2.saveGameDir);end;
		fileCopy = MultiOverlayV2.settingsDir.. "clientSettings.xml";
		saveFile = MultiOverlayV2.saveGameDir.. "clientSettings.xml";
		if not fileExists(saveFile) then
			copyFile(fileCopy, saveFile);
		end;
		moGeneralSettings.saveGameClientSettings = true;
	elseif xmlType == "objectSettings" then
		if not fileExists(MultiOverlayV2.saveGameDir) then createFolder(MultiOverlayV2.saveGameDir);end;
		fileCopy = MultiOverlayV2.settingsDir.. "objectSettings.xml";
		saveFile = MultiOverlayV2.saveGameDir.. "objectSettings.xml";
		if not fileExists(saveFile) then
			copyFile(fileCopy, saveFile);
		end;
		moGeneralSettings.saveGameObjectSettings = true;
	elseif xmlType == "language" then
		if not fileExists(MultiOverlayV2.saveGameDir) then createFolder(MultiOverlayV2.saveGameDir);end;
		fileCopy = MultiOverlayV2.settingsDir.. "language".. MultiOverlayV2.language.. ".xml";
		saveFile = MultiOverlayV2.saveGameDir.. "language".. MultiOverlayV2.language.. ".xml";
		if not fileExists(saveFile) then
			copyFile(fileCopy, saveFile);
		end;
		moGeneralSettings.saveGameLanguage = true;
	end;	
end;